;--------------------------------------------------------------------------
;  crt0.s - Generic crt0.s for a Z80
;
;  Copyright (C) 2000, Michael Hope
;
;--------------------------------------------------------------------------

	.module crt0
	.globl	_main

	.area	_HEADER (ABS)
	;; Reset vector
	.org 	0
	jp	init

	.org	0x08
	ei
	reti
	.org	0x10
	ei
	reti
	.org	0x18
	ei
	reti
	.org	0x20
	ei
	reti
	.org	0x28
	ei
	reti
	.org	0x30
	ei
	reti
	.org	0x38
	ei
	reti
	
	.org	0x66
nmi:
	jp _NMI_InterruptHandler

	.org	0x70
init:
	;; Set stack pointer directly above top of memory.
	ld	sp,#0xFFFF

	;; Initialise global variables. 
	call	gsinit

	call	_main

1$:
	halt
	jr	1$
	
;; IM2 Vektortabelle
;; 0x00 0 0 0 0 0 0 0 0 CTC Ch0
;; 0x02 0 0 0 0 0 0 1 0 CTC Ch1
;; 0x04 0 0 0 0 0 1 0 0 CTC Ch2
;; 0x06 0 0 0 0 0 1 1 0 CTC Ch3
;; 0x10 0 0 0 1 0 0 0 0 PIO A
;; 0x12 0 0 0 1 0 0 1 0 PIO B
;; 0x20 0 0 1 0 0 0 0 0 ChB Transmit Buffer Empty
;; 0x22 0 0 1 0 0 0 1 0 ChB External/Status Change
;; 0x24 0 0 1 0 0 1 0 0 ChB Receive Character Available
;; 0x26 0 0 1 0 0 1 1 0 ChB Special Receive Condition
;; 0x28 0 0 1 0 1 0 0 0 ChA Transmit Buffer Empty
;; 0x2A 0 0 1 0 1 0 1 0 ChA External/Status Change
;; 0x2C 0 0 1 0 1 1 0 0 ChA Receive Character Available
;; 0x2E 0 0 1 0 1 1 1 0 ChA Special Receive Condition
	.org 0x100
im2_table:
	.org 0x0102
	.dw _ctc_ch1_isr
	.org 0x0128
	.dw _sioa_tx_isr
	.org 0x012A
	.dw _sioa_ext_isr
	.org 0x012C
	.dw _sioa_rx_isr
	.org 0x012E
	.dw _sioa_srx_isr

	;; Ordering of segments for the linker.
	.area	_HOME
	.area	_CODE
	.area	_INITIALIZER
	.area   _GSINIT
	.area   _GSFINAL

	.area	_DATA
	.area	_INITIALIZED
	.area	_BSEG
	.area   _BSS
	.area   _HEAP

	.area   _GSINIT
gsinit::

	; Default-initialized global variables.
        ld      bc, #l__DATA
        ld      a, b
        or      a, c
        jr      Z, zeroed_data
        ld      hl, #s__DATA
        ld      (hl), #0x00
        dec     bc
        ld      a, b
        or      a, c
        jr      Z, zeroed_data
        ld      e, l
        ld      d, h
        inc     de
        ldir
zeroed_data:

	; Explicitly initialized global variables.
	ld	bc, #l__INITIALIZER
	ld	a, b
	or	a, c
	jr	Z, gsinit_next
	ld	de, #s__INITIALIZED
	ld	hl, #s__INITIALIZER
	ldir

gsinit_next:

	.area   _GSFINAL
	ret