#include "monitor.h"
#include "textlcd.h"
#include "circbuff.h"
#include <string.h>
#include <stdio.h>

unsigned char counter = 0;
unsigned long seconds = 0;

circbuff_t cb_sioa_rx, cb_sioa_tx;

/*
	9,8304 MHz
	1 Taktzyklus = 0,1017 us
	9830,4 Taktzyklen
	26 Taktzyklen mit Sprung
	379 Durchläufe
*/
void delay1ms(void) {
	__asm__ (
		"ld bc, #379         \n"
		"delay_loop:         \n"
		"	dec bc           \n"
		"	ld a, b          \n"
		"	or c             \n"
		"	jr nz, delay_loop  "
	);
}

void delay(unsigned long ms) {
	for (unsigned long i = 0; i < ms; i++)
		delay1ms();
}

bool sioa_transmit_int(void) {
	__asm__ (" di ");
	volatile unsigned char value;
	bool status = circbuff_read(&cb_sioa_tx, &value);
	if (status) 
		SIO_A_DATA = value;
	__asm__ (" ei ");
	return status;
}

// CTC Ch1 ISR
void ctc_ch1_isr(void) __interrupt {
	__asm__ ( "di" );
	counter++;
	if (counter == 200) {
		counter = 0;
		seconds++;
	}
	__asm__ ( "ei" );
}

// SIO A Transmit Buffer Empty ISR
void sioa_tx_isr(void) __interrupt {
	__asm__ ( "di" );
	volatile unsigned char value;
	if (circbuff_read(&cb_sioa_tx, &value)) {
		SIO_A_DATA = value;
	} else {
		SIO_A_CMD = 0x28; // 0b00101000 Reset Tx Pending
	}
	__asm__ ( "ei" );
}

// SIO A Receive Character Available ISR
void sioa_rx_isr(void) __interrupt {
	__asm__ ( "di" );
    circbuff_write(&cb_sioa_rx, SIO_A_DATA);
	__asm__ ( "ei" );
}

// SIO A External/Status Change
void sioa_ext_isr(void) __interrupt {
	__asm__ ( "di" );
	SIO_A_CMD = 0x10; // 0b00010000 Reset External/Status Interrupts
	__asm__ ( "ei" );
}

// SIO A Special Receive Condition
void sioa_srx_isr(void) __interrupt {
	__asm__ ( "di" );
	SIO_A_CMD = 0x30; // 0b00110000 Error Reset
	__asm__ ( "ei" );
}

void NMI_InterruptHandler(void) __critical __interrupt {
	__asm__ ( "ei" );
}

void waitTx(void) {
	unsigned char status;
	do {
		status = SIO_A_CMD;
	} while (!(status & 0x04));
}

void waitRx(void) {
	unsigned char status;
	do {
		status = SIO_A_CMD;
	} while (!(status & 0x01));
}

unsigned char readChar(void) {
	waitRx();
	return SIO_A_DATA;
}

void sendChar(unsigned int ch) {
	waitTx();
	SIO_A_DATA = ch;
}

void wait(void) {
	delay(1000);
}

void main(void) {
	__asm__ (" di ");
	KIO_CMD = 0x0B;

	circbuff_init(&cb_sioa_rx);
	circbuff_init(&cb_sioa_tx);
	
	// CTC CH0 Baudrate 19200
	unsigned char ch0_time_constant = 2;
	CTC_CH0 = 0x25; // 0b0010 0101
	CTC_CH0 = ch0_time_constant;
	
	// CTC CH1 Systick 5 ms
	unsigned char ch1_time_constant = 192;
	//CTC_CH1 = 0x25; // 0b0010 0101
	CTC_CH1 = 0xA7; // 0b1010 0101 Interrupt enabled
	CTC_CH1 = ch1_time_constant;
	
	// CTC Interrupt Vektor muss in CH0 geschrieben werden
	CTC_CH0 = 0x00;

	// SIO A mit Interrupts
	SIO_B_CMD = 0x02; // WR2
	SIO_B_CMD = 0x20; // Interrupt Vektor
	SIO_B_CMD = 0x01; // WR1
	SIO_B_CMD = 0x04; // 0b00000100 Status affects Vector
	// SIO A Standard Init
	SIO_A_CMD = 0x18; // 0b00011000
	SIO_A_CMD = 0x04; // WR4
	SIO_A_CMD = 0x04; // 0x00000100 1xClock, 1 Stopbit, keine Parität
	SIO_A_CMD = 0x05; // WR5
	SIO_A_CMD = 0x68; // 0b01101000 Tx 8 Datenbits, Tx Enable
	SIO_A_CMD = 0x03; // WR3
	SIO_A_CMD = 0xC1; // 0b11000001 Rx 8 Datenbits, Rx Enable
	// SIO A Interrupt Enable
	SIO_A_CMD = 0x01; // WR1
	SIO_A_CMD = 0x1B; // 0b00011011 Interrupt Enable

	// PIO A
	PIO_A_CMD = 0xCF; // 0b11001111 Mode 3
	PIO_A_CMD = 0x00; // Output
	
	// Init Interrupts
	__asm__ (
        "ld a, #0x01 \n"
        "ld i, a     \n"
		"im 2        \n"
		"ei          "
	);
	
	LCD_Init();
	LCD_SetCursor(LCD_LINE1);
	
	const unsigned char text = "Hello Zilog Z80";
	int textlen = 15;
	for (int i = 0; i < textlen; i++) {
		circbuff_write(&cb_sioa_tx, text[i]);
		LCD_Data(text[i]);
	}
	circbuff_write(&cb_sioa_tx, 13); circbuff_write(&cb_sioa_tx, 10);
	sioa_transmit_int();

	PIA_C_CMD = 0x00;
	unsigned char leds = 0xAA;
	const char* hexDigits = "0123456789ABCDEF";
	
	while (1) {
		if (!circbuff_empty(&cb_sioa_rx)) {
			unsigned char value;
			circbuff_read(&cb_sioa_rx, &value);
			circbuff_write(&cb_sioa_tx, value);
			sioa_transmit_int();
		}

		LCD_SetCursor(LCD_LINE2);
		char test[20];
		
		unsigned long secs = seconds;
		unsigned char hours = secs / 3600;
		secs %= 3600;
		unsigned char mins = secs / 60;
		secs %= 60;
		
		sprintf(test, "%02u:%02u:%02u R %2u T %2u", hours, mins, secs, cb_sioa_rx.num_entries, cb_sioa_tx.num_entries);
		for (int i = 0; i < strlen(test); i++)
			LCD_Data(test[i]);
				
		/*
		sendChar(hexDigits[(counter >> 4) & 0x0F]);
		sendChar(hexDigits[counter & 0x0F]);
		sendChar('-');
		sendChar(hexDigits[(seconds >> 4) & 0x0F]);
		sendChar(hexDigits[seconds & 0x0F]);
		sendChar('-');
		sendChar(13); sendChar(10);
		*/
		/*
		PIA_C_DATA = leds;
		leds = ~leds;
		*/
		// delay(1000);
	}

	/* ------------------------------
	unsigned char ch = 0;
	while (ch != 13) {
		ch = readChar();
		sendChar(ch);
		LCD_Data(ch);
	}
	sendChar(13); sendChar(10);
	--------------------------------*/
	
	/* ------------------------------
	// LED Lauflicht
	PIA_C_CMD = 0x00;
	unsigned char leds = 1;
	while (1) {
		for (int i = 0; i < 7; i++) {
			PIA_C_DATA = leds;
			leds = leds << 1;
			wait();
		}
		for (int i = 0; i < 7; i++) {
			PIA_C_DATA = leds;
			leds = leds >> 1;
			wait();
		}
	}
	---------------------------------*/
}